package test;

import java.util.HashMap;
import java.util.Map;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.api.jms.HornetQJMSClient;
import org.hornetq.api.jms.JMSFactoryType;
import org.hornetq.core.remoting.impl.netty.NettyConnectorFactory;
import org.hornetq.core.remoting.impl.netty.TransportConstants;


/** Demo */
public class Demo {

	public static void main(String[] args) throws Exception {
		produce();
		try {
			consume(true); // Consume messages that fail on consumer side
		}
		catch (Exception ex) {
			System.out.println("Sleeping some seconds, please restart server");
			Thread.sleep(1000L * 30L);
		}

		consume(false);
	}


	private static void produce() throws JMSException {
		Connection connection = getConnectionFactory().createConnection();
		// Send Messages
		Session sessionProduce = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);
		MessageProducer producer = sessionProduce.createProducer(HornetQJMSClient.createQueue("testing"));
		for (int i = 1; i <= 50; i++) {
			TextMessage message = sessionProduce.createTextMessage("Entry " + i);
			message.setObjectProperty("JMSXGroupID", "mygroup");
			producer.send(message);
			System.out.println("Send message: " + message.getText());
		}
	}


	private static void consume(boolean fail) throws Exception {
		Connection connection = getConnectionFactory().createConnection();
		Session sessionConsumer = connection.createSession(false, Session.CLIENT_ACKNOWLEDGE);
		MessageConsumer consumer = sessionConsumer.createConsumer(HornetQJMSClient.createQueue("testing"));
		connection.start();
		try {
			while(true) {
				Thread.sleep(300);
				TextMessage message = (TextMessage)consumer.receive();
				if (fail) {
					// dispatching message failed for whatever reasons (the consuming client simply has an error processing the message)
					// now try to restart the session, so failed messages won't be acknowledged.
					System.out.println("Failing: " + message.getText());
					//sessionConsumer.close();
					//sessionConsumer = connection.createSession(false, Session.CLIENT_ACKNOWLEDGE);
					//consumer = sessionConsumer.createConsumer(HornetQJMSClient.createQueue("testing"));
					sessionConsumer.recover(); // seems to be same as the three lines above (?)
					// recover seems to be the counterpart of acknowledge and is required, otherwise unacknowledged failed messages will 
				}
				else {
					System.out.println("Received message: " + message.getText());
					message.acknowledge();
				}
			}
		}
		finally {
			connection.close();
		}
	}


	public static ConnectionFactory getConnectionFactory() {
		Map<String, Object> connectionParams = new HashMap<String, Object>();
		connectionParams.put(TransportConstants.HOST_PROP_NAME, "localhost");
		connectionParams.put(TransportConstants.PORT_PROP_NAME, 5445);
		TransportConfiguration transportConfiguration = new TransportConfiguration(NettyConnectorFactory.class.getName(), connectionParams);
		return (ConnectionFactory)HornetQJMSClient.createConnectionFactoryWithoutHA(JMSFactoryType.CF, transportConfiguration);
	}

}
